;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                     ;;;
;;;  Aby Louw                                                           ;;;
;;;                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                      ;;
;;;  A generic voice definition file for a multisyn synthesiser          ;;
;;;  Customized for: csir_isizulu_buhle                                  ;;
;;;                                                                      ;;
;;;                                                                      ;;
;;;                                                                      ;;
;;;                                                                      ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; get multysin library dir


;; SYSTEM DEPENDENT PATHS in case you want to provide absolute paths to data (TO FIX)
;;(defvar csir_isizulu_buhle_dir1 "/var/source/isizulu")
;;(defvar csir_isizulu_buhle_dir "/var/source/isizulu/db")

;; Relative paths to data

(defvar csir_isizulu_buhle_dir1 (path-append libdir "../../isizulu/csir_isizulu_buhle_multisyn/"))
(defvar csir_isizulu_buhle_dir (path-append libdir "../../isizulu/csir_isizulu_buhle_multisyn/db/"))

;; SCHEME PATHS

(defvar multisyn_lib_dir (path-append libdir "multisyn/"))
(defvar kw_multisyn_dir (cdr (assoc 'csir_isizulu_buhle_multisyn voice-locations)))
(if (not (member_string multisyn_lib_dir libdir))
  (set! load-path (cons multisyn_lib_dir load-path)))

(set! load-path (cons (path-append csir_isizulu_buhle_dir1 "files_scm/") 
		      load-path))


;;; Ensure this version of festival has been compiled with multisyn module
(require 'multisynCSIR) ;; runtime scheme support


;;;  Add the directory contains general voice stuff to load-path
(set! load-path (cons (path-append csir_isizulu_buhle_dir "files_scm/") 
		      load-path))

;;; Voice specific parameter are defined in each of the following
;;; files
(require 'csir_isizulu_buhle_phoneset)
(require 'csir_isizulu_buhle_tokenizer)
(require 'csir_isizulu_buhle_tagger)
(require 'csir_isizulu_buhle_lexicon)
(require 'csir_isizulu_buhle_phrasing)
(require 'csir_isizulu_buhle_intonation)
(require 'csir_isizulu_buhle_duration)
(require 'csir_isizulu_buhle_f0model)
(require 'csir_isizulu_buhle_other)
(require 'csir_isizulu_buhle_phone_conversion)
(require 'csir_isizulu_buhle_diphone_backoff)
(require 'csir_isizulu_buhle_target_cost)
;; ... and others as required

;;;  Code specific to the multisyn waveform synthesis method

;;; You may wish to change this (only used in building the voice)
(set! csir_isizulu_buhle::closest_voice 'voice_kal_diphone)

;;;  These are the parameters which are needed at run time
;;;  build time parameters are added to his list in csir_isizulu_buhle_build.scm
(set! csir_isizulu_buhle::dt_params
      (list
       (list 'db_dir csir_isizulu_buhle_dir)
       '(name csir_isizulu_buhle)
       '(srate 16000)
       '(ob_beam 0)
       '(p_beam 0)
       
       ;;; Voice database files

;;(defvar utts_file ufile)


;(format t (quote utts_file))
    

;;; Voice pauses files
 
       ;;; Flite specific stuff
       '(out_type B) ; Output voice params as Binary[B] of C-type[C] files
    ;;   '(flite_dir "../../isizulu/db/flite/")

       '(debug_level 1)
))

(define (csir_isizulu_buhle::voice_reset)
  "(csir_isizulu_buhle::voice_reset)
Reset global variables back to previous voice."
  (csir_isizulu_buhle::reset_phoneset)
  (csir_isizulu_buhle::reset_tokenizer)
  (csir_isizulu_buhle::reset_tagger)
  (csir_isizulu_buhle::reset_lexicon)
  (csir_isizulu_buhle::reset_phrasing)
  (csir_isizulu_buhle::reset_intonation)
  (csir_isizulu_buhle::reset_duration)
  (csir_isizulu_buhle::reset_f0model)
  (csir_isizulu_buhle::reset_other)
  (csir_isizulu_buhle::reset_backoff_rules)
  (csir_isizulu_buhle::reset_target_cost)
  t
)

;; This function is called to setup a voice.  It will typically
;; simply call functions that are defined in other files in this directory
;; Sometime these simply set up standard Festival modules othertimes
;; these will be specific to this voice.
;; Feel free to add to this list if your language requires it


(define (csir_isizulu_buhle::domain_specific_voice_setup)
  "(csir_isizulu_buhle::domain_specific_voice_setup)
Define voice for limited domain: isizulu."
  ;; *always* required
  (voice_reset)


  ;; Select appropriate phone set
  (csir_isizulu_buhle::select_phoneset)

  ;; Select appropriate tokenization
  (csir_isizulu_buhle::select_tokenizer)

  ;; For part of speech tagging
  (csir_isizulu_buhle::select_tagger)

  (csir_isizulu_buhle::select_lexicon)
  ;; For multisyn selection you probably don't want vowel reduction
  ;; the multisyn selection will do that
  (if (string-equal "americanenglish" (Param.get 'Language))
      (set! postlex_vowel_reduce_cart_tree nil))

  (csir_isizulu_buhle::select_phrasing)

  (csir_isizulu_buhle::select_intonation)

  (csir_isizulu_buhle::select_duration)

  (csir_isizulu_buhle::select_f0model)

  (csir_isizulu_buhle::select_backoff_rules)

  (csir_isizulu_buhle::select_target_cost)

  ;; This is where you can modify power (and sampling rate) if desired
  (set! after_synth_hooks nil)
;  (set! after_synth_hooks
;      (list
;        (lambda (utt)
;          (utt.wave.rescale utt 2.1))))

  (set! current_voice_reset csir_isizulu_buhle::voice_reset)

  (set! current-voice 'csir_isizulu_buhle_multisyn)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (voice_csir_isizulu_buhle_multisyn)
  "(voice_csir_isizulu_buhle_multisyn)
 Set up the current voice to be female British English (Nina) using
 the standard multisynection code.  The data file UTTSFILE is used 
  when constructing the voice (i.e. which utterace structure files are
 loaded into the database."
  (voice_reset)

  (csir_isizulu_buhle::domain_specific_voice_setup)
  
  (Param.set 'Synth_Method 'MultiSyn)
  (Param.set 'unisyn.window_symmetric 0)
  

(set! csir_isizulu_buhle_dirs (list (path-append  csir_isizulu_buhle_dir "utts/")
			   (path-append csir_isizulu_buhle_dir "lpc/")
			   (path-append csir_isizulu_buhle_dir "lpc/")
			   (path-append csir_isizulu_buhle_dir "coef_stripped/")
			   ".utt" ".res" ".lpc" ".coef"))



(set! module_list (list (list  csir_isizulu_buhle_dirs "utts.data")))
;;			     (list eno_pauses_dirs "utts.pauses")))

;;  (set! module_list (list (list csir_isizulu_buhle_dirs (cadr (assoc_string 'utts_file csir_isizulu_buhle::dt_params)))))
 
  ;; if seperate pauses were recorded, add them to voice
;;  (if (cadr (assoc_string 'pauses_utts_file csir_isizulu_buhle::dt_params))
  ;;    (begin
;;	(set! module_list (list (car module_list) (list csir_isizulu_buhle_dirs (cadr (assoc_string 'pauses_utts_files csir_isizulu_buhle::dt_params)))))))
  

  (set! csir_isizulu_buhle_multisyn_voice 
	(multisyn_load_voice_modules 'csir_isizulu_buhle_multisyn (cadr (assoc_string 'srate csir_isizulu_buhle::dt_params))
			       csir_isizulu_buhle_backoff_rules (cadr (assoc_string 'db_dir csir_isizulu_buhle::dt_params)) 
			       module_list csir_isizulu_buhle_tc (cadr (assoc_string 'ob_beam csir_isizulu_buhle::dt_params))
			       (cadr (assoc_string 'p_beam csir_isizulu_buhle::dt_params))))
  


  (set! current-voice csir_isizulu_buhle_multisyn_voice)
  (set! currentMultiSynVoice csir_isizulu_buhle_multisyn_voice)
  (voice.debugLevel csir_isizulu_buhle_multisyn_voice (cadr (assoc_string 'debug_level csir_isizulu_buhle::dt_params)))
  )



(proclaim_voice
 'csir_isizulu_buhle_multisyn
 '((language isiZulu)(gender male)(dialect main)
   (description "buhle multisyn unitsel voice (default configuration).")))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(provide 'csir_isizulu_buhle_multisyn)

