;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                     ;;;
;;; Aby Louw                                                            ;;;
;;;                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                     ;;;
;;; Code for building data for prompts, aligning and unit selection     ;;;
;;; synthesizer (Multisyn unit selection synthesiser)                   ;;;
;;;                                                                     ;;;
;;; This file is only used at database build time                       ;;;
;;;                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar csir_isizulu_buhle::unitsel_dir ".")

;;; Basic voice definition file with voice defines and multisyn
;;; parameter definitoin for run time.
(load "festvox/csir_isizulu_buhle_multisyn.scm")

(defvar closure_regex "cl")
(defvar sp_threshold 0.060) ; was 0.050 for utts 5, was 0.02 for 5b
(defvar sp_closure_threshold 0.100) 

(define (utt.synth_build_promts_domain utt)
  (Initialize utt)
  (Text utt)
  (Token_POS utt)    
  (Token utt)
  (POS utt)
  (Phrasify utt)
  (Word utt)
  (Pauses utt))

(define (utt.synth_build_promts_closest_voice utt)
  (Intonation utt)
  (PostLex utt)
  (Duration utt)
  (Int_Targets utt)
  (Wave_Synth utt))


(define (convert_phones_to_closest_voice utt)
  ;; replace phones with radio_phoneset equivalent and store orig name as a feature
  (let ((segments (utt.relation.items utt 'Segment)))
    (mapcar
     (lambda (l)
       (item.set_feat l "orig_name" (item.name l))
       (item.set_name l (cadr (assoc_string (item.name l) csir_isizulu_buhle::phone_conversion ))))
     segments))
)

(define (convert_phones_to_voice utt)
  ;; get back original phones
  (let ((segments (utt.relation.items utt 'Segment)))
    (mapcar
     (lambda (l)
       (item.set_name l (item.feat l "orig_name")))
     segments))
)

(define (do_prompt_new_voice name text) 
  "(do_prompt name text) 
Synthesize given text and save waveform and labels for prompts."
  (let ((utt1 (utt.synth_build_promts_domain (eval (list 'Utterance 'Text text)))))
    (convert_phones_to_closest_voice utt1)
    (utt.save utt1 (format nil "prompt-utt/%s.utt" name))
    )
  )

(define (do_prompt_closest_voice name text) 
  "(do_prompt name text) 
Synthesize given text and save waveform and labels for prompts."
  (let ((utt1 (eval (list 'Utterance 'Text text))))
    (utt.load utt1 (format nil "prompt-utt/%s.utt" name))
    (utt.synth_build_promts_closest_voice utt1)
    (convert_phones_to_voice utt1)
    (utt.save utt1 (format nil "prompt-utt/%s.utt" name))
    (utt.save.segs utt1 (format nil "prompt-lab/%s.lab" name))
    (utt.save.wave utt1 (format nil "prompt-wav/%s.wav" name))
    )
  
  )

(define (build_prompts file)
  "(build_prompt file) 
For each utterances in prompt file, synth and save waveform and
labels for prompts and aligning."
  (voice_reset)
  (csir_isizulu_buhle::domain_specific_voice_setup)
  (let ((p (load file t)))
    (mapcar
     (lambda (l)
       (format t "%s\n" (car l))
       (do_prompt_new_voice (car l) (cadr l))
       t)
     p)
    t)
  (voice_reset)
  (eval (list csir_isizulu_buhle::closest_voice))
  (let ((p (load file t)))
    (mapcar
     (lambda (l)
       (format t "%s\n" (car l))
       (do_prompt_closest_voice (car l) (cadr l))
       t)
     p)
    t)
)


(define (do_prompt_multisyn_voice name text) 
  "(do_prompt_multisyn_voice name text) 
Synthesize given text and save waveform and labels for prompts."
  (let ((utt1 (utt.synth (eval (list 'Utterance 'Text text)))))
    ;;fix end's
    ;;;;;;;;;;;;;
    (let ((segments (utt.relation.items utt1 'Segment)))
      (mapcar
       (lambda (l)
	 (item.set_feat l "end" (item.feat l "source_end")))
       segments))
    ;;;;;;;;;;;;;;
    (utt.save utt1 (format nil "prompt-utt/%s.utt" name))
    (utt.save.segs utt1 (format nil "prompt-lab/%s.lab" name))
    (utt.save.wave utt1 (format nil "prompt-wav/%s.wav" name))))


(define (build_prompts_multisyn_voice file)
  "(build_prompts_multisyn_voice file) 
For each utterances in prompt file, synth and save waveform and
labels for prompts and aligning."
  (voice_reset)
  (voice_csir_isizulu_buhle_multisyn)
  (let ((p (load file t)))
    (mapcar
     (lambda (l)
       (format t "%s\n" (car l))
       (do_prompt_multisyn_voice (car l) (cadr l))
       t)
     p)
    t)
)



(define (do_synth name text ofd)
  "(do_synth name text ofd)
Synthesize given text and save waveform and labels for prompts."
  (let ((utt1 (utt.synth_toSegment (eval (list 'Utterance 'Text text)))))
    (utt.save.mlf utt1 (format nil "%s.lab" name) ofd)))

(define (utt.synth_toSegment utt)
  (Initialize utt)
  (Text utt)
  (Token_POS utt)    ;; when utt.synth is called
  (Token utt)
  (POS utt)
  (Phrasify utt)
  (Word utt)
  (Pauses utt)
  (Intonation utt)
  (PostLex utt))


(define (build_utts file)
  "(build_utts file) 
For each utterances in prompt file, synthesize and merge aligned labels
to predicted labels building a new utterances and saving it."
  (csir_isizulu_buhle::domain_specific_voice_setup)
  (let ((p (load file t)))
    (mapcar
     (lambda (l)
       (format t "%s\n" (car l))
       (align_utt (car l) (cadr l))
       t)
     p)
    t))

 
(define (align_utt name text)
  "(align_utts file) 
Synth an utterance and load in the actualed aligned segments and merge
them into the synthesizer utterance."
  (let ((utt1 (utt.synth_toSegment (eval (list 'Utterance 'Text text))))
	(silence (car (cadr (car (PhoneSet.description '(silences))))))
	segments actual-segments pms)
    (set! pms (track.load (format nil "pm/%s.pm" name)))
    (utt.relation.load utt1 'actual-segment 
		       (format nil "lab/%s.lab" name))
    (set! segments (utt.relation.items utt1 'Segment))
    (set! actual-segments (utt.relation.items utt1 'actual-segment))

;    (format t "       Segments are: %l\n" (mapcar (lambda (x) (item.name x)) segments))
;    (format t "Actual Segments are: %l\n" (mapcar (lambda (x) (item.name x)) actual-segments))

    ;; These should align, but if the labels had to be hand edited
    ;; then they may not. 
    ;; If you need to something more elaborate you'll have to change the
    ;; code below.
    (while (and segments actual-segments)
   ;   (format t "\ncurrent seg:%l\n" (item.name (car segments)))
   ;   (format t "current actual seg:%l\n" (item.name (car actual-segments)))
   ;   (format t "test1 = %s\n" (not (string-equal (item.name (car segments))
;				(item.name (car actual-segments)))))
;      (format t "test2 = %s\n" (string-equal (item.name (car segments)) silence))
      (cond
       ;; PROCESS STOP AND AFFRICATE CLOSURES IN ACTUAL-SEGMENTS.
       ((and (string-equal (item.name (car segments))
			   (item.name (car actual-segments)))
	     (string-matches (item.name (car actual-segments)) closure_regex))
;	(format t "PROCESS STOP AND AFFRICATE\n")
	(item.set_feat (cadr actual-segments) 
		       "cl_end" 
		       (item.feat (car actual-segments) "end"))
	(item.delete (car actual-segments))
	(item.delete (car segments))
	(set! actual-segments (cdr actual-segments))
	(set! segments (cdr segments)))
       ;; FIX WIERD SILENCES NAMES IN ACTUAL-SEGMENTS
       ((and (not (string-equal (item.name (car actual-segments)) silence))
	     (or (string-equal (item.name (car actual-segments)) "sil")
		 (string-equal (item.name (car actual-segments)) "sp") ;hkt small pause
		 (string-equal (item.name (car actual-segments)) "H#")
		 (string-equal (item.name (car actual-segments)) "h#")))
;	(format t " Fixing silence name.\n")
	(item.set_feat (car actual-segments) "name" silence))
       ;; INSERT MISSMATCHED SILENCES THAT APPEAR IN ACTUAL-SEGMENTS
       ((and (not (string-equal (item.name (car segments))
				(item.name (car actual-segments))))
	     (string-equal (item.name (car actual-segments)) silence))
;	(format t " mismatched silence from actual segments.\n")
	(if (not (or (and (item.prev (car actual-segments))
			  (< (- (parse-number (item.feat (car actual-segments) "end"))
				(parse-number (item.feat (item.prev (car actual-segments)) "end")))
			     sp_threshold))
		     (and (item.prev (car actual-segments))
			  (item.next (car actual-segments))
			  (string-matches (item.name (item.next (car actual-segments))) closure_regex)
			  (< (- (parse-number (item.feat (car actual-segments) "end"))
				(parse-number (item.feat (item.prev (car actual-segments)) "end")))
			     sp_closure_threshold))))

	    (begin
	;      (format t "inserting silence from actual segments.\n")
	      (item.insert (car segments)
			   (list silence (list (list "end" (item.feat (car actual-segments) "end"))
					       (list "score" (normalise_score (car actual-segments)))
					       (list "start" "F:standard+unisyn_start")))
			   'before)))
	(set! actual-segments (cdr actual-segments)))
       ;; DELETE MISSMATCHED SILENCES THAT OCCUR IN SEGMENTS
     ;  (print silence)
       ((and (not (string-equal (item.name (car segments))
				(item.name (car actual-segments))))
	     (string-equal (item.name (car segments)) silence))
;	(format t "deleting pause in current seg\n")
	(item.delete (car segments))
	(set! segments (cdr segments)))
       ;; DELETE SILENCES LESS THAT 50MS IN LENGTH FROM ACTUAL SEGMENTS
       ;((and (string-equal (item.name (car actual-segments)) silence)
	;     (item.prev (car actual-segments))
	 ;    (< (- (parse-number (item.feat (car actual-segments) "end"))
		;   (parse-number (item.feat (item.prev (car actual-segments)) "end")))
		;0.050))
	;(format t " Deleting short silence.\n")
	;(item.delete (car actual-segments))
	;(item.delete (car segments))
	;(set! actual-segments (cdr actual-segments))
	;(set! segments (cdr segments)))
       ;; VOWEL REDUCTION SUBSTITUTION
       ((and (not (string-equal (item.name (car segments))
				(item.name (car actual-segments))))
	     (and (string-equal (item.feat (car segments) 'ph_vc) "+")
		  (string-equal (item.feat (car actual-segments) 'ph_vlng) "a")))
	(if (eq? (parse-number (item.feat (car segments) "R:SylStructure.parent.stress")) 0)    ; old version
;	(if (eq? (parse-number (item.feat (car segments) "reducable")) 1)
;	(if t   ; lets try doing this anyway. 
	    (begin
	      ;(format t " %f vowel reduction: %s -> %s lex reducable: %s\n" 
		;      (item.feat (car actual-segments) 'end)
		 ;     (item.name (car segments)) 
		  ;    (item.name (car actual-segments)) 
		   ;   (item.feat (car segments) 'reducable))
	      (item.set_feat (car segments) "name" (item.name (car actual-segments))))
	    (begin
	      ;(format t " %f vowel reduction blocked : (%s -> %s)   lex reducable: %s\n"
		;      (item.feat (car actual-segments) 'end)
		 ;     (item.name (car segments)) 
		  ;    (item.name (car actual-segments)) 
		   ;   (item.feat (car segments) 'reducable))
	      (item.set_feat (car actual-segments) "name" (item.name (car segments))))))
       ;; SAFE SEGMENT NAME TO UNDO
       ;; MATCHING SEGMENT
       ((string-equal (item.name (car segments))
		      (item.name (car actual-segments)))
;	(format t "matching segment\n")
	(item.set_feat (car segments) "end" (item.feat (car actual-segments) "end"))
	(item.set_feat (car segments) "score" (normalise_score (car actual-segments)))
	(item.set_feat (car segments) "start" "F:standard+unisyn_start")
	; copy cl_end for stops
	(if (> (item.feat (car actual-segments) "cl_end") 0)
	    (item.set_feat (car segments) "cl_end" (item.feat (car actual-segments) "cl_end")))
	; add a feature for dipthongs
	(if (string-equal (item.feat (car segments) "ph_vlng") "d")
	    (item.set_feat (car segments) "dipth" 1))
	; check for at least one pitchmark
	(if (not (check_pms (car actual-segments) pms))
	    (begin
	      (format t "Bad pitchmarking: %s %s.\n" (item.feat (car segments) "name") (item.feat (car segments) "end"))
	      (item.set_feat (car segments) "bad" "no_pms")))
	(set! segments (cdr segments))
	(set! actual-segments (cdr actual-segments)))
       ;; ALIGNMENT MISSMATCH ERROR
       (t
	(format stderr
		"align missmatch at %s (%f) %s (%f)\n"
		(item.name (car segments))
		(item.feat (car segments) "end")
		(item.name (car actual-segments))
		(item.feat (car actual-segments) "end"))
	(error)))
      )
    (mapcar
     (lambda (seg)
      ;; shorten and split sliences
      (while (and (string-equal (item.name seg) silence)
		  (> (item.feat seg "segment_duration") 0.300))
;              (format t "splitting %s silence of %f at %f\n"
;		      (item.name seg)
;                      (item.feat seg "segment_duration")
;                      (item.feat seg "end"))
              (cond
               ((string-equal "h#" (item.feat seg "p.name"))
                (item.set_feat (item.prev seg) "end"
                               (+ 0.150 (item.feat seg "p.end"))))
               ((and (string-equal silence (item.feat seg "p.name"))
                     (string-equal silence (item.feat seg "p.p.name")))
                (item.set_feat (item.prev seg) "end"
                               (+ 0.150 (item.feat seg "p.end")))
                (item.set_feat (item.prev seg) "name" silence))
               (t
                (item.insert seg
                             (list silence
                                   (list 
                                    (list "end" (+ 0.150 
						   (item.feat seg "p.end")))
				    (list "score" (item.feat seg "score"))
				    (list "start" "F:standard+unisyn_start")))
                             'before)))))
     (utt.relation.items utt1 'Segment))

    (utt.relation.delete utt1 'actual-segment)
    (utt.set_feat utt1 "fileid" name)
    ;; If we have an F0 add in targets too
    (if (probe_file (format nil "f0/%s.f0" name))
	(build::add_targets utt1))
    (utt.save utt1 (format nil "festival/utts/%s.utt" name))
    t))


(define (check_pms seg pms)
  "(check_pms seg pms)
Check there is at least one pitchmark in this segment. Or synthesis will fail!"
  (let (start end)
    (set! end (item.feat seg "end"))
    (if (item.prev seg)
	(set! start (item.feat seg "start"))
	(set! start 0))
    (if (equal? (track.index_below pms start) (track.index_below pms end))
	nil
	t)))

(define (normalise_score seg)
"(normalise_score score)
Normalise alignmnet score by length."
(let (start end)
  (set! end (item.feat seg "end"))
  (if (item.prev seg)
      (set! start (item.feat seg "start"))
      (set! start 0))
  (exp (/ (parse-number (item.feat seg "score")) (- end start)))))


(define (add_duration_info_utts file durfile)
  "(add_duration_info utts file durfile) 
Load average durations info and for each utterances in prompt file mark outliers."
  (let ((uttslist (load file t))
	(durslist (load durfile t))
	(utt (Utterance Text ""))
	mean sd log_seg_dur)
    (mapcar
     (lambda (li)
       (format t "%s\n" (car li))
       (utt.load utt (format nil "utt/%s.utt" (car li)))
       ;; process utterance
       (mapcar
	(lambda (seg)
	  (if (assoc_string (item.feat seg "name") durslist)
	      (begin
		(set! mean (car (cadr (assoc_string (item.feat seg "name") durslist))))
		(set! sd   (car (cdr (cadr (assoc_string (item.feat seg "name") durslist)))))
		(set! log_seg_dur (log (- (item.feat seg "end") (item.feat seg "start"))))
		;(format t "seg: %s dur: %f mean: %f sd: %f range: %f:%f\n" (item.feat seg "name") (exp log_seg_dur) mean sd (exp (- mean (* 2 sd))) (exp (+ mean (* 3 sd))))
		(if (or (> log_seg_dur (+ mean (* 3 sd)))
			(< log_seg_dur (- mean (* 2 sd))))
		    (begin
		      ;(format t "duration outlier\n")
		      (item.set_feat seg "bad_dur" "outlier")))))
	  t)
	(utt.relation.items utt 'Segment))
       (utt.save utt (format nil "utt/%s.utt" (car li)))
       t)
     uttslist)
    t))
