;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                     ;;;
;;;                     Carnegie Mellon University                      ;;;
;;;                  and Alan W Black and Kevin Lenzo                   ;;;
;;;                      Copyright (c) 1998-2000                        ;;;
;;;                        All Rights Reserved.                         ;;;
;;;                                                                     ;;;
;;; Permission is hereby granted, free of charge, to use and distribute ;;;
;;; this software and its documentation without restriction, including  ;;;
;;; without limitation the rights to use, copy, modify, merge, publish, ;;;
;;; distribute, sublicense, and/or sell copies of this work, and to     ;;;
;;; permit persons to whom this work is furnished to do so, subject to  ;;;
;;; the following conditions:                                           ;;;
;;;  1. The code must retain the above copyright notice, this list of   ;;;
;;;     conditions and the following disclaimer.                        ;;;
;;;  2. Any modifications must be clearly marked as such.               ;;;
;;;  3. Original authors' names are not deleted.                        ;;;
;;;  4. The authors' names are not used to endorse or promote products  ;;;
;;;     derived from this software without specific prior written       ;;;
;;;     permission.                                                     ;;;
;;;                                                                     ;;;
;;; CARNEGIE MELLON UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK        ;;;
;;; DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     ;;;
;;; ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT  ;;;
;;; SHALL CARNEGIE MELLON UNIVERSITY NOR THE CONTRIBUTORS BE LIABLE     ;;;
;;; FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES   ;;;
;;; WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN  ;;;
;;; AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,         ;;;
;;; ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF      ;;;
;;; THIS SOFTWARE.                                                      ;;;
;;;                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Phonset for csir_isizulu
;;;

;;;  Feeel free to add new feature values, or new features to this
;;;  list to make it more appropriate to your language

;; This is where it'll fall over if you haven't defined a 
;; a phoneset yet, if you have, delete this, if you haven't
;; define one then delete this error message


(defPhoneSet
  csir_isizulu
  ;;;  Phone Features
  (
   ;; vowel, consonant, diacritic, silence, closure or other
   (vc + - d s cl 0)  
   
   ;; vowel length: shrt long diphtong schwa
   (vlng s l d a 0)

   ;; vowel height: high mid low
   (vheight 1 2 3 0)
   
   ;; vowel frontness: front mid back
   (vfront 1 2 3 0)
   
   ;; lip rounding
   (vrnd + - 0)
   
   ;; consonant types : [s] stop (plosives), [f] fricative, [a] affricate, 
   ;;                   [a(l/r/g/o)] approximant(lateral/retroflex/gluide/other), [n] nasal, 
   ;;                   [c] clicks, [t] tap/flap, [r] trill, [v] voiced implosives, 
   ;;                   [sa] stop with aspiration, [ca] click with aspiration, [la] "lateral affricate",
   ;;                   [nc] nasalized clicks

   (ctype s f a al ar ag ao n c t r v sa ca la nc 0)

   ;; place of articulation: [b] bilabial, [i] interdental, [a] alveolar, [z] alveopalatal, 
   ;;                        [p]  palatal, [l] labiodental, [d] dental, [v] velar, 
   ;;                        [g] glottal, [x] pharyngeal, [u] uvular,  
   ;;                        [r] retroflex 
   (cplace b i a z p l d v g x u r 0)

   ;; consonant voicing
   (cvox + - 0)
   )
  ;; Phone set members
  (
   ;; (ph   vc  vl  vh  vf  vr  ct  cp  cv)
   
   ;; CONSONANTS (pulmonic)

   ;; Stops or Plosives 
   ( p      -   0   0   0   0   s   b   -)             ; IPA 112
   ( b      -   0   0   0   0   s   b   +)             ; IPA 98
   ( t      -   0   0   0   0   s   a   -)             ; IPA 116
   ( d      -   0   0   0   0   s   a   +)             ; IPA 100
   ( k      -   0   0   0   0   s   v   -)             ; IPA 107
   ( g      -   0   0   0   0   s   v   +)             ; IPA 103

   ;; Stops with aspiration
   ( P      -   0   0   0   0   sa  b   -)             ; IPA 112
   ( B      -   0   0   0   0   sa  b   +)             ; IPA 98
   ( T      -   0   0   0   0   sa  a   -)             ; IPA 116
   ( K      -   0   0   0   0   sa  v   -)             ; IPA 107
   ( kg     -   0   0   0   0   sa  v   +)             ; HACK !!! "soft" k: voiced implosive?

   ;; Nasals
   ( m      -   0   0   0   0   n   b   +)             ; IPA 109
   ( n      -   0   0   0   0   n   a   +)             ; IPA 110 
   ( ng     -   0   0   0   0   n   v   +)             ; ng
   ( nj     -   0   0   0   0   n   p   +)             ; IPA 110 

   ;; Tril
   ( r      -   0   0   0   0   r   a   +)             ; IPA 114

   ;; Fricative
   ( f      -   0   0   0   0   f   l   -)             ; IPA 102
   ( v      -   0   0   0   0   f   l   +)             ; IPA 118
   ( s      -   0   0   0   0   f   a   -)             ; IPA 115
   ( z      -   0   0   0   0   f   a   +)             ; IPA 122
   ( G      -   0   0   0   0   f   v   -)             ; IPA 120
   ( h      -   0   0   0   0   f   g   -)             ; IPA 104 
   ( H      -   0   0   0   0   f   g   +)             ; HACK!!! actually breathy
   ( sh     -   0   0   0   0   f   p   -)             ; 

   ;; APPROXIMANTS

   ;; Lateral
   ( hl      -   0   0   0   0   al  a   -)             ; HACK!!! Lateral fricative
   ( l      -   0   0   0   0   al  a   +)             ; IPA 108       ;; Lateral approximants 
   
   ;; Glide
   ( j      -   0   0   0   0   ag  p   +)            ; IPA 106 
   ( w      -   0   0   0   0   ag  v   +)            ; IPA 119


   ;; CONSONANTS (non-pulmonic)

   ;; Clicks 
   ( q      -   0   0   0   0   c   d   -)             ; IPA 45                                     |\
   ( c      -   0   0   0   0   c   a   -)             ; IPA 60                                     !\
   ( x      -   0   0   0   0   c   p   -)             ; IPA 248                                    =\
   ( qv      -   0   0   0   0   c   d   +)
   ( cv      -   0   0   0   0   c   a   +)
   ( xv      -   0   0   0   0   c   p   +)

   ;; Clicks with aspiration
   ( Q      -   0   0   0   0   ca  d   -)             ; IPA 45                                     |\
   ( C      -   0   0   0   0   ca  a   -)             ; IPA 60                                     !\
   ( X      -   0   0   0   0   ca  p   -)             ; IPA 248                                    =\
   
   ;; VOWELS
   ( i      +   l   1   1   -   0   0   0)             ; IPA 121                                    y
   ( e      +   l   1   2   -   0   0   0)             ; IPA 232                                    }
   ;;( E      +   l   2   2   -   0   0   0)             ; IPA ???                                    }
   ;;( O      +   s   2   3   +   0   0   0)             ; IPA 111
   ( o      +   s   2   2   +   0   0   0)             ; 
   ( u      +   s   3   1   +   0   0   0)             ; IPA 207                                    {
   ( a      +   l   3   3   -   0   0   0)             ; IPA 191                                    V
   ( ey     +   l   1   2   -   0   0   0)
   
   ;Affricate
   ( tsh    -   0   0   0   0   a   p   -)             ; IPA 116 + IPA 83  (t  + sh)
   ( ts     -   0   0   0   0   a   a   -)             ; IPA 100 + IPA 90  (t  + s)
   ;;( dz     -   0   0   0   0   a   a   +)             ;   (d  + z)
   ( jh     -   0   0   0   0   a   p   +)             ; 

   ;"Lateral Affricate" - A hack!!!
   ( kl      -   0   0   0   0   la  a   -)             ; IPA 241       
   ( dl     -   0   0   0   0   la   p   +)             ; IPA 116 + IPA 83  (d  + zh)

   ;; SILENCES
   ( pau    s   0   0   0   0   0   0   0)             ; Silence
   ( #      s   0   0   0   0   0   0   0)             ; Silence
   ( H#     s   0   0   0   0   0   0   0)             ; Silence
   ( cl     cl  0   0   0   0   0   0   0)             ; Closure
   ( gs     s   0   0   0   0   0   0   0)             ; Glottal stop
   )
  )

(PhoneSet.silences '(pau # H#))

(define (csir_isizulu_buhle::select_phoneset)
  "(csir_isizulu_buhle::select_phoneset)
Set up phone set for csir_isizulu."
  (Parameter.set 'PhoneSet 'csir_isizulu)
  (PhoneSet.select 'csir_isizulu)
)

(define (csir_isizulu_buhle::reset_phoneset)
  "(csir_isizulu_buhle::reset_phoneset)
Reset phone set for csir_isizulu."
  t
)

(provide 'csir_isizulu_buhle_phoneset)


